/****************************************************************************
** libebml : parse EBML files, see http://embl.sourceforge.net/
**
** <file/class description>
**
** Copyright (C) 2002-2003 Steve Lhomme.  All rights reserved.
**
** This file is part of libmatroska.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding an other license may use this file in accordance with 
** the Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.matroska.org/license/qpl/ for QPL licensing information.
** See http://www.matroska.org/license/gpl/ for GPL licensing information.
**
** Contact license@matroska.org if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

/*!
	\file
	\version \$Id: EbmlUnicodeString.cpp,v 1.6 2003/04/28 21:55:19 robux4 Exp $
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/

#include <clocale>
#include <cassert>

#include "StdInclude.h"
#include "EbmlUnicodeString.h"

START_LIBEBML_NAMESPACE

EbmlUnicodeString::EbmlUnicodeString()
 :EbmlElement(0, false)
{
	DefaultSize = 0;
	setlocale(LC_CTYPE, "UTF-8"); // done but not ensured :(
}

EbmlUnicodeString::EbmlUnicodeString(const UTFstring & aDefaultValue)
 :EbmlElement(0, true), Value(aDefaultValue), DefaultValue(aDefaultValue)
{
	DefaultSize = 0;
	DefaultIsSet = true;
	setlocale(LC_CTYPE, "UTF-8"); // done but not ensured :(
}

EbmlUnicodeString::EbmlUnicodeString(const EbmlElement & ElementToClone)
 :EbmlElement(0, false)
{
	*(reinterpret_cast<EbmlElement*>(this)) = ElementToClone;
}

EbmlUnicodeString::EbmlUnicodeString(const EbmlElement & ElementToClone, const UTFstring & aDefaultValue)
 :EbmlElement(0, true), Value(aDefaultValue), DefaultValue(aDefaultValue)
{
	*(reinterpret_cast<EbmlElement*>(this)) = ElementToClone;
	DefaultIsSet = true;
}

/*!
	\todo handle conversion
	\todo handle exception on errors
*/
uint32 EbmlUnicodeString::RenderData(IOCallback & output, bool bSaveDefault)
{
	uint32 Result = 0;
	
	char * tmpStr = new char[2*Value.length()];
	std::string UTF8String;
	
	if (tmpStr != NULL) {
#if defined(NO_WSTRING)
	    UTF8String = Value;
#else
	    ::wcstombs(tmpStr, &Value[0], 2*Value.length());
#endif

		UTF8String = tmpStr;

		output.writeFully(UTF8String.c_str(), UTF8String.length());
		Result = UTF8String.length();

		if (UTF8String.length() < DefaultSize) {
			// pad the rest with 0
			binary *Pad = new binary[DefaultSize - UTF8String.length()];
			if (Pad != NULL) {
				memset(Pad, 0x00, DefaultSize - UTF8String.length());
				output.writeFully(Pad, DefaultSize - UTF8String.length());
				delete Pad;

				Result = DefaultSize;
			}
		}

		delete tmpStr;
	}

	return Result;
}

EbmlUnicodeString & EbmlUnicodeString::operator=(const UTFstring NewString)
{
	Value = NewString;
	bValueIsSet = true;
	return *this;
}

#if !defined(NO_WSTRING)
/*!
	\todo Decode UTF-8 encoded to Wide char string (internal)
*/
EbmlUnicodeString & EbmlUnicodeString::operator=(const std::string NewString)
{
	wchar_t *tmpStr = new wchar_t[2*NewString.length()];
	if (tmpStr != NULL) {
		::mbstowcs(tmpStr, NewString.c_str(), 2*NewString.length());
		Value = tmpStr;
		delete tmpStr;
	}
	bValueIsSet = true;

	return *this;
}
#endif // NO_WSTRING

uint64 EbmlUnicodeString::UpdateSize(bool bSaveDefault)
{
	if (!bSaveDefault && IsDefaultValue())
		return 0;

	char * tmpStr = new char[2*Value.length()];
	std::string UTF8String;
	Size = 0;
	
	if (tmpStr != NULL) {
#if defined(NO_WSTRING)
	    UTF8String = Value;
#else
	    ::wcstombs(tmpStr, &Value[0], 2*Value.length());
#endif
		UTF8String = tmpStr;
		Size = UTF8String.length();
		if (Size < DefaultSize) {
			Size = DefaultSize;
		}
	}
	
	return Size;
}

uint64 EbmlUnicodeString::ReadData(IOCallback & input)
{
	char *Buffer = new char[Size+1];
	if (Buffer == NULL) {
		// impossible to read, skip it
		input.setFilePointer(Size, seek_current);
	} else {
		input.readFully(Buffer, Size);
		if (Buffer[Size-1] != '\0') {
			Buffer[Size] = '\0';
		}
		
		wchar_t *tmpStr = new wchar_t[2*Size];
		::mbstowcs(tmpStr, Buffer, 2*Size);
		Value = tmpStr;
		delete tmpStr;
		delete Buffer;
	}

	return Size;
}

END_LIBEBML_NAMESPACE
